#' Rescale a precipitation data product in time
#'
#' The function \code{mon_to_year} aggregates the requested data set from monthly to yearly time steps and stores it in <project_folder>/data/processed. If the input data set is a Raster object the output will be stored in the same location of the input file.
#'
#' @importFrom methods as is
#' @param name a Raster object or a character string with the name(s) of the desired data set. Suitable options are:
#' \itemize{
#' \item{"20cr" for 20CR v3,}
#' \item{"chirps" for CHIRPS v2.0,}
#' \item{"cmap" for CMAP standard version,}
#' \item{"cmorph" for CMORPH,}
#' \item{"cpc" for CPC-Global,}
#' \item{"cru-ts" for CRU_TS v4.06,}
#' \item{"em-earth" for EM-EARTH,}
#' \item{"era20c" for ERA-20C,}
#' \item{"era5" for ERA5,}
#' \item{"ghcn" for GHCN-M v2,}
#' \item{"gldas-clsm" for GLDAS CLSM,}
#' \item{"gldas-noah" for GLDAS NOAH,}
#' \item{"gldas-vic" for GLDAS VIC,}
#' \item{"gpcc" for GPCC v2020,}
#' \item{"gpcp" for GPCP v2.3,}
#' \item{"gpm_imerg" for GPM IMERGM Final v06,}
#' \item{"mswep" for MSWEP v2.8,}
#' \item{"ncep-doe" for NCEP/DOE,}
#' \item{"ncep-ncar" for NCEP/NCAR,}
#' \item{"persiann" for PERSIANN-CDR,}
#' \item{"precl" for PREC/L,}
#' \item{"terraclimate" for TerraClimate,}
#' \item{"trmm-3b43" for TRMM 3B43 v7,}
#' \item{"udel" for UDEL v501.}
#' }
#' @param database_path a character string with the path where the "database" folder is located.
#' @return No return value, called to subset via cdo
#' @export
#' @examples
#' \dontrun{
#' x <- mon_to_year("gpcp", tempdir())
#' w <- raster::brick("dummie.nc")
#' z <- mon_to_year(w, tempdir())
#' }

mon_to_year <- function(name, database_path = "./data/database"){
  if (!Reduce("&", is.element(name, c("all", "20cr", "chirps", "cmap", "cmorph", "cpc", "cru-ts", "em-earth", "era20c", "era5", "ghcn", "gldas-clsm", "gldas-noah", "gldas-vic", "gpcc", "gpcp", "gpm-imerg", "mswep", "ncep-doe", "ncep-ncar", "persiann", "precl", "terraclimate", "trmm-3b43", "udel")))){
    if (is(name, "RasterBrick") | is(name, "RasterLayer") | is(name, "RasterStack")){
      nc_in <- name@file@name
      nc_out <- sub(".nc.*", "", nc_in)
      nc_out <- paste0(nc_out, "_yearly.nc")
      cdo_str <- paste0("cdo --no_warnings -L -z zip_4 -setmon,1 -setday,1 -yearsum ", nc_in, " ", nc_out)
      system(cdo_str)
      return(invisible())
    } else {
      stop("Error: Data set not available. Select from 20cr, chirps, cmap, cmorph, cpc, cru-ts, em-earth, era20c, era5, ghcn, gldas-clsm, gldas-noah, gldas-vic, gpcc, gpcp, gpm-imerg, mswep, ncep-doe, ncep-ncar, persiann, precl, terraclimate, trmm-3b43, udel")
    }
  }
  if (!grepl("*/data/database", database_path)){
    stop("Error: database_path should point to the location of '<project_folder>/data/database'")
  }
  nc_in <- grep(name, list.files(database_path, full.names = TRUE), value = TRUE)
  start_date <- substr(nc_in, nchar(nc_in) - 27, nchar(nc_in) - 22)
  start_year <- substr(start_date, 1, 4)
  start_month <- substr(start_date, 5, 6)
  end_date <- substr(nc_in, nchar(nc_in) - 20, nchar(nc_in) - 15)
  end_year <- substr(end_date, 1, 4)
  end_month <- substr(end_date, 5, 6)
  if ((as.numeric(start_month) != 1) & (as.numeric(end_month) != 12)){
    nc_out <- substr(nc_in, 1, nchar(nc_in) - 29)
    nc_out <- gsub("database", "processed", nc_out)
    nc_out <- paste(nc_out, as.numeric(start_year) + 1, as.numeric(end_year) - 1, "025_yearly.nc", sep = "_")
    cdo_str <- paste0("cdo --no_warnings -L -z zip_4 -setmon,1 -setday,1 -yearsum -delete,year=", start_year, ",", end_year, " ", nc_in, " ", nc_out)
  } else if ((as.numeric(start_month) != 1) & (as.numeric(end_month) == 12)){
    nc_out <- substr(nc_in, 1, nchar(nc_in) - 29)
    nc_out <- gsub("database", "processed", nc_out)
    nc_out <- paste(nc_out, as.numeric(start_year) + 1, end_year, "025_yearly.nc", sep = "_")
    cdo_str <- paste0("cdo --no_warnings -L -z zip_4 -setmon,1 -setday,1 -yearsum -delete,year=", start_year, " ", nc_in, " ", nc_out)
  } else if ((as.numeric(start_month) == 1) & (as.numeric(end_month) != 12)){
    nc_out <- substr(nc_in, 1, nchar(nc_in) - 29)
    nc_out <- gsub("database", "processed", nc_out)
    nc_out <- paste(nc_out, start_year, as.numeric(end_year) - 1, "025_yearly.nc", sep = "_")
    cdo_str <- paste0("cdo --no_warnings -L -z zip_4 -setmon,1 -setday,1 -yearsum -delete,year=", end_year, " ", nc_in, " ", nc_out)
  } else {
    nc_out <- substr(nc_in, 1, nchar(nc_in) - 29)
    nc_out <- gsub("database", "processed", nc_out)
    nc_out <- paste(nc_out, start_year, end_year, "025_yearly.nc", sep = "_")
    cdo_str <- paste0("cdo --no_warnings -L -z zip_4 -setmon,1 -setday,1 -yearsum ", nc_in, " ", nc_out)
  }
  system(cdo_str)
  return(invisible())
}